// SPDX-FileCopyrightText: 2023-2024 Laurent Montel <montel.org>
//
// SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL

#include "KLLMConstants.h"

QUrl KLLMCore::ollamaUrl()
{
    return QUrl(QStringLiteral("http://127.0.0.1:11434"));
}
